#  Created byMartin.cz
#  Copyright (c) Martin Strohalm. All rights reserved.

import ui
from ... enums import *


UI_LINE_CAP = {
    LINE_CAP_BUTT: ui.LINE_CAP_BUTT,
    LINE_CAP_SQUARE: ui.LINE_CAP_SQUARE,
    LINE_CAP_ROUND: ui.LINE_CAP_ROUND}

UI_LINE_JOIN = {
    LINE_JOIN_BEVEL: ui.LINE_JOIN_BEVEL,
    LINE_JOIN_MITER: ui.LINE_JOIN_MITER,
    LINE_JOIN_ROUND: ui.LINE_JOIN_ROUND}

UI_LINE_STYLE = {
    LINE_STYLE_CUSTOM: LINE_STYLE_CUSTOM,
    LINE_STYLE_SOLID: LINE_STYLE_SOLID,
    LINE_STYLE_DOTTED: DASH_VALUES_DOTTED,
    LINE_STYLE_DASHED: DASH_VALUES_DASHED,
    LINE_STYLE_DASHDOTTED: DASH_VALUES_DASHDOTTED}

UI_FILL_RULE = {
    FILL_RULE_EVENODD: True,
    FILL_RULE_WINDING: False}

UI_FONT_STYLE = {
    FONT_STYLE_NORMAL: FONT_STYLE_NORMAL,
    FONT_STYLE_ITALIC: FONT_STYLE_ITALIC}

UI_FONT_WEIGHT = {
    FONT_WEIGHT_NORMAL: FONT_WEIGHT_NORMAL,
    FONT_WEIGHT_LIGHT: FONT_WEIGHT_LIGHT,
    FONT_WEIGHT_BOLD: FONT_WEIGHT_BOLD,
    FONT_WEIGHT_BLACK: FONT_WEIGHT_BLACK,
    FONT_WEIGHT_HEAVY: FONT_WEIGHT_HEAVY,
    FONT_WEIGHT_SEMIBOLD: FONT_WEIGHT_SEMIBOLD,
    FONT_WEIGHT_MEDIUM: FONT_WEIGHT_MEDIUM,
    FONT_WEIGHT_ULTRALIGHT: FONT_WEIGHT_ULTRALIGHT,
    FONT_WEIGHT_THIN: FONT_WEIGHT_THIN}

UI_FONT_FAMILY = {
    FONT_FAMILY_SERIF: FONT_FACE_SERIF,
    FONT_FAMILY_SANS: FONT_FACE_SANS,
    FONT_FAMILY_MONO: FONT_FACE_MONO}

UI_TOUCH_STATE = {
    'began': TOUCH_PRESSED,
    'ended': TOUCH_RELEASED,
    'moved': TOUCH_MOVED,
    'stationary': TOUCH_STATIC,
    'cancelled': TOUCH_RELEASED}
