#  Created byMartin.cz
#  Copyright (c) Martin Strohalm. All rights reserved.

# import main objects
from . color import Color, COLORS
from . palette import Palette, PALETTES
from . gradient import Gradient, GRADIENTS

# register named objects
from . named import *
