#  Created byMartin.cz
#  Copyright (c) Martin Strohalm. All rights reserved.

import py5
from ... enums import *


PY5_LINE_CAP = {
    LINE_CAP_BUTT: py5.PROJECT,
    LINE_CAP_ROUND: py5.ROUND,
    LINE_CAP_SQUARE: py5.SQUARE}

PY5_LINE_JOIN = {
    LINE_JOIN_MITER: py5.MITER,
    LINE_JOIN_ROUND: py5.ROUND,
    LINE_JOIN_BEVEL: py5.BEVEL}
