#  Created byMartin.cz
#  Copyright (c) Martin Strohalm. All rights reserved.

# define values
NONE = 'none'

DEG = 'deg'
RAD = 'rad'

LEFT = 'left'
RIGHT = 'right'
TOP = 'top'
BOTTOM = 'bottom'
CENTER = 'center'
MIDDLE = 'middle'
START = 'start'
END = 'end'

BEFORE = 'before'
AFTER = 'after'

EVENODD = 'evenodd'
WINDING = 'winding'

N = 'n'
NW = 'nw'
NE = 'ne'
S = 's'
SW = 'sw'
SE = 'se'
W = 'w'
E = 'e'
C = 'c'

HORIZONTAL = 'h'
VERTICAL = 'v'

INSIDE = 'inside'
OUTSIDE = 'outside'

NATURAL = 'natural'
FACEIN = 'facein'
FACEOUT = 'faceout'
FOLLOW = 'follow'

AUTO = 'auto'
CUSTOM = 'custom'

TRANS = 'trans'
SOLID = 'solid'
DOTTED = '.'
DASHED = '-'
DASHDOTTED = '-.'

BUTT = 'butt'
SQUARE = 'square'
BEVEL = 'bevel'
MITER = 'miter'
ROUND = 'round'

NORMAL = 'normal'
ITALIC = 'italic'
LIGHT = 'light'
BOLD = 'bold'
BLACK = 'black'
HEAVY = 'heavy'
SEMIBOLD = 'semibold'
MEDIUM = 'medium'
ULTRALIGHT = 'ultralight'
THIN = 'thin'

SERIF = 'serif'
SANS = 'sans'
MONO = 'mono'

JSON = 'json'
SVG = 'svg'
QT = 'qt'
WX = 'wx'
CAIRO = 'cairo'
MUPDF = 'mupdf'
PYTHONISTA = 'pythonista'
PY5 = 'py5'

DAYS = 'd'
HOURS = 'h'
MINUTES = 'm'
SECONDS = 's'
MSECONDS = 'ms'
USECONDS = 'us'
NSECONDS = 'ns'

CEIL = 'ceil'
FLOOR = 'floor'
HALFUP = 'halfup'
